import React, { Component } from 'react';
import logo from './logo.svg';
import './App.css';

import {Router, Route, Link, Switch} from "react-router-dom";
import createHistory from "history/createBrowserHistory";

import {createStore, combineReducers} from 'redux';
import {Provider, connect}   from 'react-redux';

import { GraphQLClient } from 'graphql-request'

const gql = new GraphQLClient("http://localhost:4000/graphql", { headers: {} })

gql.request(`query getPosts {
  posts {
    id
    title
    text
    timestamp
    tagz
  }
}`).then(data => store.dispatch({type: "DATA", data}))

function postsReducer(state, action)
{
    if (action.type === 'DATA'){
        console.log(action.data)
        return {data: action.data.posts, status: 'DATA'}
    }
    return {data: [], status: 'EMPTY'}
}


const reducers = combineReducers({
    posts: postsReducer,
})

var store = createStore(reducers);
//store.subscribe( () => console.log(store.getState()))
//
//
//

class TagList extends Component {
    render() {
        return (
            <div className="tagList">
                {this.props.tags.map(tag => <span>{tag}</span>)}
            </div>
        )
    }
}

class FeedPost extends Component {
    render(){
        console.log(this.props)
        return (
            <div className='post'>
                <h1>{this.props.post.title}</h1>
                <div>{this.props.post.text}</div>
                <div>{(new Date(this.props.post.timestamp *1000)).toLocaleString()}</div>
                <TagList tags={this.props.post.tagz} />
                <Link to={`/post/${this.props.post.id}`}>more...</Link>
            </div>
        );
    }
}

class Feed extends Component {
    render(){
        if (this.props.posts.status === 'EMPTY'){
            return (
            <div className='feedContainer'>
                Loading.....
            </div>
            )
        }
        return (
            <div className='feedContainer'>
                {this.props.posts.data.map(item => <FeedPost post={item} key={item.id} />)}
            </div>
        );
    }
}


const mapStateToProps = function(store) {
  return {
    posts: store.posts,
  };
}

Feed = connect(mapStateToProps)(Feed)

let FeedPage = (props) => <Feed />

class Post extends Component {
    render(){
        return (
            <div>
                one post with id = {this.props.id}
            </div>
        )
    }
}

let PostPage = (props) => <Post id={props.match.params.id}/>

class App extends Component {
  render() {
    return (
        <Provider store={store}>
            <Router history={createHistory()} >
                <Switch>
                    <Route path='/' component={FeedPage} exact />
                    <Route path='/post/:id' component={PostPage} />
                </Switch>
            </Router>
        </Provider>
    );
  }
}

export default App;
